@@ -110,7 +110,7 @@ static KDirWatch::Method methodFromString(const QByteArray &method)
     } else if (method == "QFSWatch") {
         return KDirWatch::QFSWatch;
     } else {
-#ifdef Q_OS_LINUX
+#if defined(HAVE_SYS_INOTIFY_H)
         // inotify supports delete+recreate+modify, which QFSWatch doesn't support
         return KDirWatch::INotify;
 #else
@@ -784,13 +784,17 @@ bool KDirWatchPrivate::useStat(Entry *e)
 
 /* If <instance> !=0, this KDirWatch instance wants to watch at <_path>,
  * providing in <isDir> the type of the entry to be watched.
- * Sometimes, entries are dependant on each other: if <sub_entry> !=0,
- * this entry needs another entry to watch himself (when notExistent).
+ * Sometimes, entries are dependent on each other: if <sub_entry> !=0,
+ * this entry needs another entry to watch itself (when notExistent).
  */
 void KDirWatchPrivate::addEntry(KDirWatch *instance, const QString &_path,
                                 Entry *sub_entry, bool isDir, KDirWatch::WatchModes watchModes)
 {
     QString path(_path);
+    if (path.startsWith(QLatin1String(":/"))) {
+        qCWarning(KDIRWATCH) << "Cannot watch QRC-like path" << path;
+        return;
+    }
     if (path.isEmpty()
 #ifndef Q_OS_WIN
             || path == QLatin1String("/dev")
@@ -850,7 +854,7 @@ void KDirWatchPrivate::addEntry(KDirWatch *instance, const QString &_path,
         if (e->isDir && !isDir) {
             qCWarning(KCOREADDONS_DEBUG) << "KDirWatch:" << path << "is a directory. Use addDir!";
         } else if (!e->isDir && isDir) {
-            qWarning("KDirWatch: %s is a file. Use addFile!", qPrintable(path));
+            qCWarning(KCOREADDONS_DEBUG) << "KDirWatch:" << path << "is a file. Use addFile!";
         }
 
         if (!e->isDir && (watchModes != KDirWatch::WatchDirOnly)) {
@@ -1017,7 +1021,7 @@ void KDirWatchPrivate::removeWatch(Entry *e)
 #endif
 }
 
-bool KDirWatchPrivate::removeEntry(KDirWatch *instance,
+void KDirWatchPrivate::removeEntry(KDirWatch *instance,
                                    const QString &_path,
                                    Entry *sub_entry)
 {
@@ -1025,12 +1029,9 @@ bool KDirWatchPrivate::removeEntry(KDirWatch *instance,
         qCDebug(KDIRWATCH) << "path=" << _path << "sub_entry:" << sub_entry;
     }
     Entry *e = entry(_path);
-    if (!e) {
-        return false;
+    if (e) {
+        removeEntry(instance, e, sub_entry);
     }
-
-    removeEntry(instance, e, sub_entry);
-    return true;
 }
 
 void KDirWatchPrivate::removeEntry(KDirWatch *instance,
@@ -1870,7 +1871,7 @@ KDirWatch::KDirWatch(QObject *parent)
 
 KDirWatch::~KDirWatch()
 {
-    if (dwp_self.hasLocalData()) { // skip this after app destruction
+    if (d && dwp_self.hasLocalData()) { // skip this after app destruction
         d->removeEntries(this);
     }
 }
@@ -1905,18 +1906,14 @@ QDateTime KDirWatch::ctime(const QString &_path) const
 void KDirWatch::removeDir(const QString &_path)
 {
     if (d) {
-        if (!d->removeEntry(this, _path, nullptr)) {
-            qCWarning(KCOREADDONS_DEBUG) << "doesn't know" << _path;
-        }
+        d->removeEntry(this, _path, nullptr);
     }
 }
 
 void KDirWatch::removeFile(const QString &_path)
 {
     if (d) {
-        if (!d->removeEntry(this, _path, nullptr)) {
-            qCWarning(KCOREADDONS_DEBUG) << "doesn't know" << _path;
-        }
+        d->removeEntry(this, _path, nullptr);
     }
 }
 
