@@ -21,6 +21,7 @@
 
 #include "kuser.h"
 #include "config-getgrouplist.h"
+#include "config-accountsservice.h"
 
 #include <QtCore/QMutableStringListIterator>
 #include <QtCore/QDir>
@@ -34,7 +35,16 @@
 #include <algorithm>  // std::find
 #include <functional> // std::function
 
-class KUser::Private : public QSharedData
+#if defined(__BIONIC__)
+static inline struct passwd * getpwent() { return nullptr; }
+inline void setpwent() { }
+static inline void setgrent() { }
+static inline struct group * getgrent() { return nullptr; }
+inline void endpwent() { }
+static inline void endgrent() { }
+#endif
+
+class Q_DECL_HIDDEN KUser::Private : public QSharedData
 {
 public:
     uid_t uid;
@@ -46,7 +56,7 @@ public:
     Private() : uid(uid_t(-1)), gid(gid_t(-1)) {}
     Private(const char *name) : uid(uid_t(-1)), gid(gid_t(-1))
     {
-        fillPasswd(name ? ::getpwnam(name) : 0);
+        fillPasswd(name ? ::getpwnam(name) : nullptr);
     }
     Private(const passwd *p) : uid(uid_t(-1)), gid(gid_t(-1))
     {
@@ -56,7 +66,11 @@ public:
     void fillPasswd(const passwd *p)
     {
         if (p) {
+#ifndef __BIONIC__
             QString gecos = QString::fromLocal8Bit(p->pw_gecos);
+#else
+            QString gecos = QString();
+#endif
             QStringList gecosList = gecos.split(QLatin1Char(','));
             // fill up the list, should be at least 4 entries
             while (gecosList.size() < 4) {
@@ -170,7 +184,16 @@ QString KUser::homeDir() const
 
 QString KUser::faceIconPath() const
 {
-    QString pathToFaceIcon(homeDir() + QDir::separator() + QStringLiteral(".face.icon"));
+    QString  pathToFaceIcon;
+    if (!d->loginName.isEmpty()) {
+        pathToFaceIcon = QStringLiteral(ACCOUNTS_SERVICE_ICON_DIR) + QLatin1Char('/') + d->loginName;
+    }
+
+    if (QFile::exists(pathToFaceIcon)) {
+        return pathToFaceIcon;
+    }
+
+    pathToFaceIcon = QString(homeDir() + QLatin1Char('/') + QStringLiteral(".face.icon"));
 
     if (QFile::exists(pathToFaceIcon)) {
         return pathToFaceIcon;
@@ -313,7 +336,7 @@ KUser::~KUser()
 {
 }
 
-class KUserGroup::Private : public QSharedData
+class Q_DECL_HIDDEN KUserGroup::Private : public QSharedData
 {
 public:
     gid_t gid;
@@ -322,7 +345,7 @@ public:
     Private() : gid(gid_t(-1)) {}
     Private(const char *_name) : gid(gid_t(-1))
     {
-        fillGroup(_name ? ::getgrnam(_name) : 0);
+        fillGroup(_name ? ::getgrnam(_name) : nullptr);
     }
     Private(const ::group *p) : gid(gid_t(-1))
     {
@@ -399,7 +422,7 @@ QString KUserGroup::name() const
     return d->name;
 }
 
-void listGroupMembers(gid_t gid, uint maxCount, std::function<void(passwd *)> handleNextGroupUser)
+static void listGroupMembers(gid_t gid, uint maxCount, std::function<void(passwd *)> handleNextGroupUser)
 {
     if (maxCount == 0) {
         return;
@@ -410,7 +433,7 @@ void listGroupMembers(gid_t gid, uint maxCount, std::function<void(passwd *)> ha
     }
     uint found = 0;
     QVarLengthArray<uid_t> addedUsers;
-    struct passwd *p = 0;
+    struct passwd *p = nullptr;
     for (char **user = g->gr_mem; *user; user++) {
         if ((p = getpwnam(*user))) {
             addedUsers.append(p->pw_uid);
